@@ -1,8 +1,7 @@
 import sys
 import dbus
-from telepathy.client import (Connection, Channel)
-from telepathy.interfaces import (CONN_INTERFACE, CHANNEL_TYPE_TUBES)
 from telepathy.constants import (CONNECTION_HANDLE_TYPE_CONTACT)
+import time
 
 from stream_tube_client import StreamTubeJoinerClient, \
         StreamTubeInitiatorClient
@@ -11,28 +10,23 @@ class StreamTubeInitiatorPrivateClient(StreamTubeInitiatorClient):
     def __init__(self, account_file, contact_id, socket_address=None):
         StreamTubeInitiatorClient.__init__(self, account_file, None, contact_id, socket_address)
 
-    def connected_cb(self):
-        StreamTubeInitiatorClient.connected_cb(self)
+    def ready_cb(self, conn):
+        StreamTubeInitiatorClient.ready_cb(self, conn)
 
-        self.tubes_with_contact()
-        self.offer_tube()
-
-    def tubes_with_contact(self):
-        handle = self.conn[CONN_INTERFACE].RequestHandles(
-                CONNECTION_HANDLE_TYPE_CONTACT, [self.contact_id])[0]
-
-        chan_path = self.conn[CONN_INTERFACE].RequestChannel(
-            CHANNEL_TYPE_TUBES, CONNECTION_HANDLE_TYPE_CONTACT,
-            handle, True)
-        self.channel_tubes = Channel(self.conn.dbus_proxy.bus_name, chan_path)
+        # Gabble will refuse to create the tube if it didn't receive contact's
+        # capability yet (to ensure that he supports tubes). Ideally we should
+        # use the ContactCapability interface to determine when we can offer
+        # the tube. As this API is still a DRAFT, we just add a delay for now.
+        time.sleep(5)
+        self.create_tube(CONNECTION_HANDLE_TYPE_CONTACT, self.contact_id)
 
 class StreamTubeJoinerPrivateClient(StreamTubeJoinerClient):
     def __init__(self, account_file, connect_trivial_client):
         StreamTubeJoinerClient.__init__(self, account_file, None, None,
                 connect_trivial_client)
 
-    def connected_cb(self):
-        StreamTubeJoinerClient.connected_cb(self)
+    def ready_cb(self, conn):
+        StreamTubeJoinerClient.ready_cb(self, conn)
 
         print "waiting for a tube offer from contacts"
 
