@@ -26,6 +26,7 @@ import weakref
 from telepathy.constants import (CONNECTION_STATUS_DISCONNECTED,
                                  CONNECTION_STATUS_CONNECTED,
                                  HANDLE_TYPE_NONE,
+                                 HANDLE_TYPE_CONTACT,
                                  LAST_HANDLE_TYPE)
 from telepathy.errors import (Disconnected, InvalidArgument,
                               InvalidHandle, NotAvailable,
@@ -335,13 +336,15 @@ class ConnectionInterfaceCapabilities(_ConnectionInterfaceCapabilities):
     @dbus.service.method(CONN_INTERFACE_CAPABILITIES, in_signature='au', out_signature='a(usuu)')
     def GetCapabilities(self, handles):
         ret = []
+        handle_type = HANDLE_TYPE_CONTACT
         for handle in handles:
-            if (handle != 0 and handle not in self._handles):
+            if (handle != 0 and (handle_type, handle) not in self._handles):
                 raise InvalidHandle
             elif handle in self._caps:
-                theirs = self._caps[handle]
-                for type in theirs:
-                    ret.append([handle, type, theirs[0], theirs[1]])
+                types = self._caps[handle]
+                for ctype, specs in types.items():
+                    ret.append([handle, ctype, specs[0], specs[1]])
+        return ret
 
     @dbus.service.signal(CONN_INTERFACE_CAPABILITIES, signature='a(usuuuu)')
     def CapabilitiesChanged(self, caps):
@@ -433,7 +436,7 @@ class ConnectionInterfaceRequests(
         type = props[CHANNEL_INTERFACE + '.ChannelType']
         handle_type = props.get(CHANNEL_INTERFACE + '.TargetHandleType',
                 HANDLE_TYPE_NONE)
-        handle = props[CHANNEL_INTERFACE + '.TargetHandle']
+        handle = props.get(CHANNEL_INTERFACE + '.TargetHandle', 0)
 
         return (type, handle_type, handle)
 
@@ -476,8 +479,12 @@ class ConnectionInterfaceRequests(
         if target_handle_type != HANDLE_TYPE_NONE:
             if target_handle == None:
                 # Turn TargetID into TargetHandle.
-                self.check_handle(target_handle_type, target_id)
-                target_handle = self._handles[target_handle_type, target_id]
+                for handle in self._handles.itervalues():
+                    if handle.get_name() == target_id and handle.get_type() == target_handle_type:
+                        target_handle = handle.get_id()
+                if not target_handle:
+                    raise InvalidHandle('TargetID %s not valid for type %d' %
+                        target_id, target_handle_type)
 
                 altered_properties[CHANNEL_INTERFACE + '.TargetHandle'] = \
                     target_handle
@@ -522,12 +529,12 @@ class ConnectionInterfaceRequests(
         in_signature='a{sv}', out_signature='boa{sv}',
         async_callbacks=('_success', '_error'))
     def EnsureChannel(self, request, _success, _error):
-        yours = not self._channel_manager.channel_exists(request)
-
         type, handle_type, handle = self._check_basic_properties(request)
         self._validate_handle(request)
         props = self._alter_properties(request)
 
+        yours = not self._channel_manager.channel_exists(props)
+
         channel = self._channel_manager.channel_for_props(props, signal=False)
 
         _success(yours, channel._object_path, props)
